/***************************************************************************
 *   Copyright (C) 2004 by Raphael Langerhorst                             *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/
 
#include "GAttractAgent.h"

#include "GPositionalInfluence.h"

#include <GObject.h>
#include <GVector3.h>
#include <GForm.h>

#include <qmutex.h>

#include <exception>

using namespace std;

using namespace GCS;

namespace GBE {

GAttractAgent::GAttractAgent(double factor, QObject *parent, const char *name)
 : GCS::GAgent(parent, name),
   Factor(factor)
{
}


GAttractAgent::~GAttractAgent()
{
}

void GAttractAgent::receiveInfluence(GElementInfluence& influence)
{
  if (requestObject()->hasForm() && requestObject()->hasEnergy())
  {
    
    try{
      GPositionalInfluence& i = dynamic_cast<GPositionalInfluence&>(influence);
      
//       qDebug("influence has position, attracting to element");
      
      //determine match...
      const GEnergy* energy = requestEnergy();
      
      double diff_level = energy->level() - i.Energy.level();
      double sum_sigma = energy->sigma() + i.Energy.sigma();
      double diff_amount = energy->amount() - i.Energy.amount();
      
      if (diff_level<0)
        diff_level = -diff_level;
        
      if (diff_amount<0)
        diff_amount = -diff_amount;
      
      if (diff_level<sum_sigma && sum_sigma>0 && energy->amount()>0 && i.Energy.amount()>0)
      {
        GForm* f = requestForm();
      
//         QMutexLocker(&f);
        
        GVector3 direction = i.position() - f->Position;
        
        if (direction.length() > 0)
        {
          //THIS formula defines attraction speed
          double diff_speed = Factor*(sum_sigma - diff_level)/sum_sigma * i.Energy.amount();
          
          direction.normalize();
          direction.mul(diff_speed);
          
  //         qDebug("calculated speed difference is " + QString::number(diff_speed));
  
          //timing mechanism: there is no timing involved because
          // it probably only depends on the amount of influence brought in;        
  
          f->Dynamics.Speed += direction;
          emit this->formChanged(*f);
        }
      }
      
    }
    catch(exception)
    {
    }
    
  }
}

};
