/***************************************************************************
 *   Copyright (C) 2004 by Raphael Langerhorst                             *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#ifndef GBEGATTRACTAGENTH
#define GBEGATTRACTAGENTH

#include <GAgent.h>

#include <GElementInfluence.h>

namespace GBE {

/**
  \class GAttractAgent GAttractAgent.h
  \brief Moves towards elements with similar energy.
  @author Raphael Langerhorst
  
  When an influence is received and the influence type is GPositionalInfluence
  then the agent accelerates towards the source of the influence. This is only
  done when a form is available, since formless elements are also positionless.
*/
class GAttractAgent : public GCS::GAgent
{
  Q_OBJECT
  
  protected:
  
    /**
     * Common acceleration factor.
     */
    double Factor;

  public:
    /**
     * Constructor
     */
    GAttractAgent(double factor = 1, QObject *parent = 0, const char *name = 0);

    /**
     * Virtual destructor for subclassing.
     */
    virtual ~GAttractAgent();
    
  public slots:
    
    /**
     * Reimplements GAgent::receiveInfluence().
     * Acceleration towards the source of the influence is implemented here.
     * The attraction force also depends on how much alike the influence
     * energy level and the own energy level are.
     * @see GCS::GEnergy, GCS::GElementInfluence.
     */
    virtual void receiveInfluence(GCS::GElementInfluence& influence);

};

};

#endif
