/***************************************************************************
 *   Copyright (C) 2004 by Raphael Langerhorst                             *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#include "GEnergyFormAgent.h"

#include "GOpenGLForm.h"

#include <GEnergy.h>
#include <GObject.h>

#include <exception>

using namespace GCS;
using namespace std;

namespace GBE {

GEnergyFormAgent::GEnergyFormAgent(double level_range_min, double level_range_max,
                                   QObject* parent, const char* name)
: GAgent(parent, name),
  LevelRangeMin(level_range_min),
  LevelRangeMax(level_range_max)
{
}


GEnergyFormAgent::~GEnergyFormAgent()
{
}

void GEnergyFormAgent::run()
{
  if (requestObject()->hasEnergy() && requestObject()->hasForm())
  {
    if (LevelRangeMax - LevelRangeMin <= 0)
    {
      qDebug("wrong range settings: min is " + QString::number(LevelRangeMin) + ", max is "  + QString::number(LevelRangeMax));
      qDebug(" falling back to default range 0 to 10");
      LevelRangeMin = 0;
      LevelRangeMax = 10;
    }
    try
    {      
      double half_range = (LevelRangeMax - LevelRangeMin)*0.5;
      double middle_value = LevelRangeMin + half_range;
      double half_range_inv = 1/half_range;
      
      while(!shutdown)
      {
        GOpenGLForm* f = dynamic_cast<GOpenGLForm*>(requestForm());
        GEnergy* e = requestEnergy();
        
        //the radius of the form should represent the amount of the energy
        f->Radius = e->amount();
        
        //we want a default value for the blue component
        f->RGBA[2] = 0.5;
        
        if (e->level() > middle_value)
        {
          f->RGBA[0] = (e->level() - middle_value)*half_range_inv;
          f->RGBA[1] = 1;
        }
        
        else
        {
          f->RGBA[0] = 0;
          f->RGBA[1] = 1-(middle_value - e->level())*half_range_inv;
        }
        
        //we update 10 times per second
        msleep(100);
      }
    }
    catch(exception e)
    {
      qDebug("form is not GOpenGLForm compatible, can't set colour or radius");
    }
    
  }
}


};
