/***************************************************************************
 *   Copyright (C) 2004 by Raphael Langerhorst                             *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#include "GInvestigationAgent.h"

#include "GOpenGLForm.h"

#include <GObject.h>
#include <GElementID.h>
#include <GElementInfluence.h>
#include <GEnergy.h>

#include <qstringlist.h>

#include <exception>

using namespace std;

using namespace GCS;

using namespace GWE;

namespace GBE {

GInvestigationAgent::GInvestigationAgent(GWEInterface* gwe, int update_interval_msec, QObject* parent, const char* name)
: GAgent(parent, name),
  KnownElementsEmitPending(false),
  Gwe(gwe),
  UpdateInterval(update_interval_msec)
{
  connect(&UpdateTimer,SIGNAL(timeout()),this,SLOT(sendChangedSignals()));
  UpdateTimer.start(1000);
}


GInvestigationAgent::~GInvestigationAgent()
{
}


void GInvestigationAgent::run()
{
  bool bEnergy = false;
  bool bForm = false;
  
  //emit interesting element data once
  
  if (requestObject()->hasEnergy())
  {
    bEnergy = true;
  }
  
  if (requestObject()->hasForm())
  {
    bForm = true;
  }
  
  if (bForm)
  {
//     double last_abs_distance;
    while (!shutdown)
    {
      GElementID target_id(this->CurrentTarget);
      const GElement* target_element = Gwe->read(target_id);
      if (target_element)
      {
        if (target_element->getObject()->hasForm())
        {
          const GForm* target_form = target_element->getObject()->getForm();
          GForm* f = requestForm();
          
          GVector3 distance = target_form->Position - f->Position;
          
          double factor = 0.002*UpdateInterval;
          if (factor>=0.95)
            factor = 0.95;
          
//           try
//           {
//             GOpenGLForm* ogl_f = dynamic_cast<GOpenGLForm*>(f);
//             double radius_diff = target_form->getRadiusMax() - ogl_f->getRadiusMax();
//             ogl_f->Radius += radius_diff*factor*0.5;
//           }
//           catch (exception e)
//           {
//           }
          
          f->Dynamics.Speed += distance.mul(factor);
        }
      }
      else
      {
        GForm* f = requestForm();
        f->Dynamics.Speed = this->CurrentSpeed;
      }
    
      msleep(UpdateInterval);
    }
  }
}

void GInvestigationAgent::sendChangedSignals()
{
  if (requestObject()->hasEnergy())
    emit this->energyChanged(*requestEnergy());
  if (requestObject()->hasForm())
    emit this->formChanged(*requestForm());
  if (this->KnownElementsEmitPending)
  {
    this->KnownElementsEmitPending = false;
    QStringList known_elements;
    for (QValueListIterator<GCS::GElementID> it = this->KnownElements.begin(); it != this->KnownElements.end(); it++)
    {
      known_elements.append(QString::number((*it).getID()));
    }
    
    emit knownElementsListChanged(known_elements);
  }
}

void GInvestigationAgent::receiveInfluence(GElementInfluence& influence)
{
  emit influenceReceived(influence);
  if (!this->KnownElements.contains(influence.source()))
  {
    this->KnownElements.append(influence.source());
    this->KnownElementsEmitPending = true;
  }
}

void GInvestigationAgent::createAndSendBasicInfluence(double fraction_of_own_energy, unsigned long targetID)
{
  if (requestObject()->hasEnergy())
  {
    if (fraction_of_own_energy > 1)
      fraction_of_own_energy = 1;
    GElementInfluence i(getElementID(),requestEnergy()->take(fraction_of_own_energy));
    if (targetID == 0)
      emit this->radiateInfluence(i);
    else
      emit this->sendInfluence(GElementID(targetID),i);
  }
}

void GInvestigationAgent::followTarget(unsigned long destinationElementID)
{
  this->CurrentTarget = destinationElementID;
}

void GInvestigationAgent::setSpeed(const GVector3& new_speed)
{
  this->CurrentSpeed = new_speed;
}

};
