/***************************************************************************
 *   Copyright (C) 2004 by Raphael Langerhorst                             *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#ifndef GBEGPLANETCATEGORY_H
#define GBEGPLANETCATEGORY_H

#include <GDynamicGeneratorCategory.h>

namespace GBE {

/**
\class GPlanetCategory GPlanetCategory.h
\brief Provides a planet category for dynamic world generation.
@author Raphael Langerhorst
*/
class GPlanetCategory : public GDynamicGeneratorCategory
{
  public:
    /**
     * Constructor.
     */
    GPlanetCategory(double range);

    /**
     * Virtual destructor.
     */
    virtual ~GPlanetCategory();

    /**
     * Creates a planet form (a sphere).
     */
    virtual GCS::GForm* createForm(const GCS::GEnergy& energy, const GCS::GVector3& position, Util::PseudoRNG& rng) const;
    
    /**
     * Create agents that should operate on the planet element.
     */
    virtual QPtrList< GCS :: GAgent > createAgents(const GCS::GEnergy& energy, Util::PseudoRNG& rng) const;

};

};

#endif
