/***************************************************************************
 *   Copyright (C) 2004 by Raphael Langerhorst                             *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/
 
#ifndef GBEGPOSITIONALINFLUENCEH
#define GBEGPOSITIONALINFLUENCEH

#include <GElementInfluence.h>

#include <GVector3.h>

namespace GBE {

/**
  \class GPositionalInfluence GPositionalInfluence.h
  \brief An influence with additional position information.

  This type of influence has a position and radius attribute added so
  agents can react according to the influence sender's position and size.
*/

class GPositionalInfluence : public GCS::GElementInfluence
{
  protected:

    /**
     * The additional position information. This should be set to
     * the position of the source element when creating the influence.
     */
    const GCS::GVector3 Position;
    
    /**
     * The additional rotation information. This should be set to
     * the rotation of the source element when creating the influence.
     */
    const GCS::GVector3 Rotation;
    
    /**
     * The additional radius information so the receiver knows about
     * size of the source element.
     */
    const double Radius;
    
    /**
     * Since the position is relative to the parent element, it could
     * be of interest to know the parent.
     */
    const GCS::GElementID Parent;
    
public:
    /**
     * Constructor.
     */
    GPositionalInfluence(const GCS::GElementID& source, const GCS::GEnergy& energy,
                         const GCS::GVector3& position,
                         const GCS::GVector3& rotation,
                         double radius,
                         const GCS::GElementID& parent);

    /**
     * Destructor.
     */
    virtual ~GPositionalInfluence();
    
    /**
     * @returns the position of the influence source.
     */
    const GCS::GVector3& position() const;
    
    /**
     * @returns the rotation of the influence source.
     */
    const GCS::GVector3& rotation() const;
    
    /**
     * @returns the radius of the influence source.
     */
    const double radius() const;
    
    /**
     * @returns the parent ID of the influence source.
     */
    const GCS::GElementID& parent() const;

};

}

#endif
