/***************************************************************************
 *   Copyright (C) 2004 by Raphael Langerhorst                             *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#include "GReparentInfluence.h"

using namespace GCS;

namespace GBE {

GReparentInfluence::GReparentInfluence(const GCS::GElementID& source, const GCS::GEnergy& energy,
                       const GCS::GElementID& reparented_element,
                       const GCS::GElementID& new_parent,
                       const GCS::GVector3& move_distance,
                       const GCS::GVector3& rotate)
: GElementInfluence(source, energy),
  ReparentedElement(reparented_element),
  NewParent(new_parent),
  MoveDistance(move_distance),
  Rotate(rotate)
{}
  

GReparentInfluence::~GReparentInfluence()
{
}

const GElementID& GReparentInfluence::getReparentedElement()
{
  return this->ReparentedElement;
}

const GElementID& GReparentInfluence::getNewParent()
{
  return this->NewParent;
}

const GVector3& GReparentInfluence::getMoveDistance()
{
  return this->MoveDistance;
}

const GVector3& GReparentInfluence::getRotate()
{
  return this->Rotate;
}

};
