/***************************************************************************
 *   Copyright (C) 2004 by Raphael Langerhorst                             *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#include "GSolarsystemForm.h"

namespace GBE {

GSolarsystemForm::GSolarsystemForm(const GCS::GVector3& position, const GCS::GVector3& rotation, double radius, double sol_size_fraction)
: GOpenGLForm(position, rotation, radius),
  SolFraction(sol_size_fraction)
{
  if (SolFraction > 1)
    SolFraction = 1;
}


GSolarsystemForm::~GSolarsystemForm()
{
}


void GSolarsystemForm::render() const
{
  GLUquadric* QuadricObject = gluNewQuadric();
  Q_ASSERT(QuadricObject);
  
  if (QuadricObject)
  {
    glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,RGBA);
    gluSphere(QuadricObject,Radius*SolFraction,16,16);
    gluQuadricNormals(QuadricObject,GLU_SMOOTH);
    gluDeleteQuadric(QuadricObject);
  }
  
  QuadricObject = gluNewQuadric();
  Q_ASSERT(QuadricObject);
  
  if (QuadricObject)
  {
    float rgba[4];
    rgba[0] = this->RGBA[0];
    rgba[1] = this->RGBA[1];
    rgba[2] = this->RGBA[2];
    rgba[3] = this->RGBA[3]*0.2;
    glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,rgba);
    gluSphere(QuadricObject,Radius,16,16);
    gluQuadricNormals(QuadricObject,GLU_SMOOTH);
    gluDeleteQuadric(QuadricObject);
  }
}

};

