/***************************************************************************
 *   Copyright (C) 2004 by Raphael Langerhorst                             *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#include "InvestigationInterfaceImp.h"

#include <qlineedit.h>
#include <qtextedit.h>
#include <qspinbox.h>
#include <qslider.h>
#include <qlistbox.h>
#include <qpushbutton.h>

using namespace GBE;

using namespace GCS;

namespace GCE
{

InvestigationInterfaceImp::InvestigationInterfaceImp(OpenGLFrame* frame, QWidget *parent, const char *name)
: InvestigationInterface(parent, name),
  ReceivedEnergyAmountSum(0),
  Frame(frame)
{
  connect(pushButtonMovementRelease,SIGNAL(clicked()),this,SLOT(releaseTargetInternal()));
  connect(&SenseTimer,SIGNAL(timeout()),this,SLOT(updateData()));
  connect(listBoxKnownElements,SIGNAL(highlighted(const QString&)),this,SLOT(setTargetIDs(const QString& )));
  SenseTimer.start(1000);
}

void InvestigationInterfaceImp::updateData()
{
  double delta_t = this->SensedEnergyAmountTimeElapsed.restart()*0.001;
  if (delta_t>0)
  {
    this->lineEditAmountPerSecond->setText(QString::number(this->ReceivedEnergyAmountSum/delta_t));
    this->ReceivedEnergyAmountSum = 0;
    
    //tell the agent its new speed:
    emit this->setSpeedInternal();
  }
}

void InvestigationInterfaceImp::setTargetIDs(const QString& s)
{
  bool ok;
  int ID = s.toInt(&ok);
  if (ok)
  {
    this->spinBoxMovementDestID->setValue(ID);
    this->spinBoxInfluenceDestID->setValue(ID);
  }
}

void InvestigationInterfaceImp::moveToInternal()
{
  unsigned long dest = spinBoxMovementDestID->value();
  emit moveTo(dest);
}

void InvestigationInterfaceImp::radiateInfluenceInternal()
{
  double fraction = (double)(sliderEnergyFraction->value())/(double)(sliderEnergyFraction->maxValue());
  emit radiateInfluence(fraction);
}

void InvestigationInterfaceImp::sendInfluenceInternal()
{
  unsigned long dest = spinBoxInfluenceDestID->value();
  double fraction = (double)(sliderEnergyFraction->value())/(double)(sliderEnergyFraction->maxValue());
  emit sendInfluence(fraction,dest);
}

void InvestigationInterfaceImp::setSpeedInternal()
{
//  bool ok = false;
//  QString s(this->lineEditPropSize->text());
//  double speed_length = s.toDouble(&ok);
//  if (ok)
//  {
    GVector3 speed = Frame->ViewProperties.ViewTarget - Frame->ViewProperties.ViewPosition;
    if (speed.length() > 0)
    {
      speed.normalize();
      speed.mul( /* speed_length* */ this->sliderMovementSpeed->value()/10);
      emit this->setSpeed(speed);
    }
//  }
}

void InvestigationInterfaceImp::releaseTargetInternal()
{
  emit moveTo(0);
}

void InvestigationInterfaceImp::influenceReceived(const GCS::GElementInfluence& influence)
{
  //DO NOT UNCOMMENT THE NEXT LINE, IT WILL CRASH THE PROGRAM BECAUSE OF TOO FREQUENT CALLS, USE BELOW qDebug() OUTPUT INSTEAD
//   this->textEditExperienceLog->append("influence received from " + QString::number(influence.source().getID()));

//   qDebug("influence received from " + QString::number(influence.source().getID()));
//   qDebug(" influence energy: " + QString::number(influence.Energy.level())
//                          + " " + QString::number(influence.Energy.amount())
//                          + " " + QString::number(influence.Energy.sigma()));

  this->ReceivedEnergyAmountSum += influence.Energy.amount();
}


void InvestigationInterfaceImp::updateForm(const GCS::GForm& form)
{
//   this->lineEditPropPosX->setUpdatesEnabled(false);
  this->lineEditPropPosX->setText(QString::number(form.Position.x));
//   this->lineEditPropPosX->setUpdatesEnabled(true);
  
//   this->lineEditPropPosY->setUpdatesEnabled(false);
  this->lineEditPropPosY->setText(QString::number(form.Position.y));
//   this->lineEditPropPosY->setUpdatesEnabled(true);
  
//   this->lineEditPropPosZ->setUpdatesEnabled(false);
  this->lineEditPropPosZ->setText(QString::number(form.Position.z));
//   this->lineEditPropPosZ->setUpdatesEnabled(true);
  
//   this->lineEditPropSize->setUpdatesEnabled(false);
  this->lineEditPropSize->setText(QString::number(form.getRadiusMax()));
//   this->lineEditPropSize->setUpdatesEnabled(true);
}


void InvestigationInterfaceImp::updateEnergy(const GCS::GEnergy& energy)
{
//   this->lineEditPropEnergyLevel->setUpdatesEnabled(false);
  this->lineEditPropEnergyLevel->setText(QString::number(energy.level()));
//   this->lineEditPropEnergyLevel->setUpdatesEnabled(true);
  
//   this->lineEditPropEnergyAmount->setUpdatesEnabled(false);
  this->lineEditPropEnergyAmount->setText(QString::number(energy.amount()));
//   this->lineEditPropEnergyAmount->setUpdatesEnabled(true);
  
//   this->lineEditPropEnergySigma->setUpdatesEnabled(false);
  this->lineEditPropEnergySigma->setText(QString::number(energy.sigma()));
//   this->lineEditPropEnergySigma->setUpdatesEnabled(true);
}

void InvestigationInterfaceImp::setKnownElementsList(const QStringList& element_list)
{
  this->listBoxKnownElements->clear();
  this->listBoxKnownElements->insertStringList(element_list);
}

}
