/***************************************************************************
 *   Copyright (C) 2004 by Raphael Langerhorst                             *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#ifndef INVESTIGATIONINTERFACEIMPH
#define INVESTIGATIONINTERFACEIMPH

#include "InvestigationInterface.h"
#include "OpenGLFrame.h"

#include <GInvestigationAgent.h>

#include <qdatetime.h>
#include <qtimer.h>
#include <qstringlist.h>
#include <qlistbox.h>

namespace GCE
{

/**
  \class InvestigationInterfaceImp InvestigationInterfaceImp.h
  \brief User interface for the InvestigationAgent.
  @author Raphael Langerhorst
*/

class InvestigationInterfaceImp : public InvestigationInterface
{
Q_OBJECT

  protected:
  
    QTime SensedEnergyAmountTimeElapsed;
    
    QTimer SenseTimer;
  
    double ReceivedEnergyAmountSum;
      
  protected:
  
    OpenGLFrame* Frame;
  
    
  //these slots only exist in the Imp class, not in InvestigationInterface itself:
  protected slots:
    virtual void updateData();
    virtual void setTargetIDs(const QString& s);
    
  
  public:
    InvestigationInterfaceImp(OpenGLFrame* frame, QWidget *parent = 0, const char *name = 0);
    
  protected slots:
    virtual void moveToInternal();
    virtual void sendInfluenceInternal();
    virtual void radiateInfluenceInternal();
    virtual void setSpeedInternal();
    virtual void releaseTargetInternal();
    
  public slots:
    virtual void influenceReceived(const GCS::GElementInfluence& influence);
    virtual void updateForm(const GCS::GForm& form);
    virtual void updateEnergy(const GCS::GEnergy& energy);
    virtual void setKnownElementsList(const QStringList& element_list);
    
//   signals:
//     virtual void moveTo(unsigned long);
//     virtual void radiateInfluence(double fraction);
//     virtual void sendInfluence(double fraction, unsigned long destID);
};

}

#endif
