/***************************************************************************
 *   Copyright (C) 2003-2004 by Raphael Langerhorst                        *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#include "GAgent.h"
#include "GForm.h"
#include "GObject.h"
#include "GEnergy.h"
#include "GElementID.h"

#include <exception>

using namespace GCS;

using namespace std;


namespace GCS
{

GAgent::GAgent(QObject* parent, const char* name)
: QObject(parent,name),
  Object(NULL),
  Agents(NULL),
  shutdown(FALSE)
{
}

GObject* GAgent::requestObject()
{
  Q_ASSERT(Object);
  if (Object)
    return Object;
  else
    throw exception();
}

void GAgent::run()
{
}

GEnergy* GAgent::requestEnergy()
{
  Q_ASSERT(Object);
  if (Object==NULL)
    throw exception();
  
  if (Object->hasEnergy() == FALSE)
    throw exception();
    
  return Object->getEnergy();
}

GForm* GAgent::requestForm()
{
  Q_ASSERT(Object);
  if (Object==NULL)
    throw exception();
  if (Object->hasForm() == FALSE)
    throw exception();
  return Object->getForm();
}

const QPtrList<GAgent>& GAgent::requestAgents() const
{
  Q_ASSERT(Agents);
  if (Agents==NULL)
    throw exception();
  return *Agents;
}

const GElementID& GAgent::getElementID() const
{
  Q_ASSERT(Object);
  if (Object==NULL)
    throw exception();
  return Object->getID();
}

const GElementID& GAgent::getConnectionID() const
{
  Q_ASSERT(Object);
  if (Object==NULL)
    throw exception();
  return Object->getConnection();
}

// SLOTS:

void GAgent::beginPark()
{
  qDebug("parking agent");
  this->shutdown = TRUE;
}

void GAgent::threadStart(double seconds_elapsed)
{
  qDebug("starting agent with delta time of " + QString::number(seconds_elapsed));
  this->start();
}

void GAgent::receiveInfluence(GCS::GElementInfluence& influence)
{
}

// public methods:

bool GAgent::isParked() const
{
  return !running();
}

}
