/***************************************************************************
 *   Copyright (C) 2004 by Raphael Langerhorst                             *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#include "GElementID.h"

namespace GCS
{

class IDRange
{
  private:
    unsigned long LowerBound;
    unsigned long UpperBound;
  
  public:
    IDRange()
    : LowerBound(0),
      UpperBound(0)
    {}
    
    IDRange(unsigned long lower_bound, unsigned long upper_bound)
    : LowerBound(lower_bound),
      UpperBound(upper_bound)
    {
      //lower bound should NEVER be 0, the 0 IDs are not valid.
      if (LowerBound <= 0)
        LowerBound = 1;
    }
    
    /**
      * @returns ID between LowerBound and UpperBound
      * @returns 0 when no ID is available (UpperBound - LowerBound <= 0)
      */
    unsigned long takeID()
    {
      if (UpperBound - LowerBound <=0)
        return 0;
      return LowerBound++;
    }
    
    void setRange(unsigned long low, unsigned long high)
    {
      LowerBound = low;
      UpperBound = high;
    }
};

IDRange GElementID::FreeIDs;
 
GElementID GElementID::getNextFreeID()
{
  return GElementID(FreeIDs.takeID());
}

void GElementID::initFreeIDs(unsigned long low, unsigned long high)
{
  FreeIDs.setRange(low,high);
}

}
