/***************************************************************************
 *   Copyright (C) 2004 by Raphael Langerhorst                             *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/

#ifndef GELEMENTINFLUENCEH
#define GELEMENTINFLUENCEH

#include "GElementID.h"
#include "GEnergy.h"

namespace GCS
{

class GElementID;
class GEnergy;

/**
  \class GElementInfluence GElementInfluence.h
  \brief GElementInfluence represents any kind of influence for elements
  @author Raphael Langerhorst
  
  Normally a whole element actually influences other elements,
  so this class should be seen as a way of transmitting influence from
  one element to another.
  
  @see GEnergy, GAgent
*/

class GElementInfluence
{
  protected:
    
    /**
     * Determines the source element of the influence.
     */
    GElementID Source;
    
  public:
  
    /**
     * The energy of the influence.
     */
    GEnergy Energy;
  
    /**
     * Constructor
     * @param source from where the influence originates.
     */
    GElementInfluence(const GElementID& source, const GEnergy& energy = GEnergy(0,0,0))
    : Source(source),
      Energy(energy)
    {}

    /**
     * Virtual destructor for subclassing.
     */
    virtual ~GElementInfluence() {}

    /**
     * @return the ID of the element that sent the influence.
     */
    const GElementID& source() const
    {
      return Source;
    }
};

}

#endif
